
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007000
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
DROLSKY
DROLSKY's
PayPal
Rolsky
Rolsky's
ARRAYn
Anno
AnnoCPAN
Branno
EXPR
Filmer
Inkster
Jens
KEYFUNC
LMU
MULTICALL
McCauley
Muey
PRs
Parseval
Purkis
Rabbitson
Rehsack
Rezic
Roode
Siegel
Signes
Slaven
Summersault
TODO
Tassilo
Tatham
Thegler
bsearchidx
de
firstidx
firstres
firstval
glitchy
lastidx
lastres
lastval
listify
minmax
natatime
notall
onlyidx
onlyres
onlyval
refactor
rehsackATcpan
thusly
uniq
von
tassilo
Adam
Kennedy
adamk
rehsack
Dave
autarch
Aaron
Crane
arc
BackPan
Brad
Forschinger
bnjf
David
Golden
dagolden
jddurand
jeandamiendurand
sno
Mash
jrmash
Ricardo
rjbs
Toby
mail
Tokuhiro
Matsuno
tokuhirom
Tom
Wyant
wyant
lib
List
SomeUtils
PP
